﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace howto_async_call_endinvoke
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Display the original images.
        private Bitmap[] Images =
        {
            Properties.Resources.Dog,
            Properties.Resources.geodesic,
            Properties.Resources.interview_puzzles,
            Properties.Resources.banner
        };
        private void Form1_Load(object sender, EventArgs e)
        {
            DisplayOriginalImages();
        }
        private void btnReset_Click(object sender, EventArgs e)
        {
            DisplayOriginalImages();
        }
        private void DisplayOriginalImages()
        {
            pictureBox1.Image = Images[0];
            pictureBox2.Image = Images[1];
            pictureBox3.Image = Images[2];
            pictureBox4.Image = Images[3];
        }

        // Emboss the images synchronously.
        private void btnSync_Click(object sender, EventArgs e)
        {
            lblElapsedTime.Text = "";
            DisplayOriginalImages();
            this.Cursor = Cursors.WaitCursor;
            Application.DoEvents();

            DateTime start_time = DateTime.Now;

            // Emboss the images.
            pictureBox1.Image = Emboss(Images[0]);
            pictureBox1.Refresh();

            pictureBox2.Image = Emboss(Images[1]);
            pictureBox2.Refresh();

            pictureBox3.Image = Emboss(Images[2]);
            pictureBox3.Refresh();

            pictureBox4.Image = Emboss(Images[3]);
            pictureBox4.Refresh();

            // Display the elapsed time.
            DateTime stop_time = DateTime.Now;
            TimeSpan elapsed_time = stop_time - start_time;
            lblElapsedTime.Text = elapsed_time.TotalSeconds.ToString("0.00") + " seconds";
            this.Cursor = Cursors.Default;
        }

        // Make a delegate representing the Emboss extension method.
        private delegate Bitmap EmbossDelegate(Bitmap bm);

        // Emboss the images asynchronously.
        private void btnAsync_Click(object sender, EventArgs e)
        {
            lblElapsedTime.Text = "";
            DisplayOriginalImages();
            this.Cursor = Cursors.WaitCursor;
            Application.DoEvents();

            DateTime start_time = DateTime.Now;

            // Copy the images.
            Bitmap bm1 = (Bitmap)Images[0].Clone();
            Bitmap bm2 = (Bitmap)Images[1].Clone();
            Bitmap bm3 = (Bitmap)Images[2].Clone();
            Bitmap bm4 = (Bitmap)Images[3].Clone();

            // Start the threads.
            EmbossDelegate caller = Emboss;
            IAsyncResult result1 = caller.BeginInvoke(bm1, null, null);
            IAsyncResult result2 = caller.BeginInvoke(bm2, null, null);
            IAsyncResult result3 = caller.BeginInvoke(bm3, null, null);
            IAsyncResult result4 = caller.BeginInvoke(bm4, null, null);

            // Wait for the threads to complete.
            pictureBox1.Image = caller.EndInvoke(result1);
            pictureBox2.Image = caller.EndInvoke(result2);
            pictureBox3.Image = caller.EndInvoke(result3);
            pictureBox4.Image = caller.EndInvoke(result4);

            // Display the elapsed time.
            DateTime stop_time = DateTime.Now;
            TimeSpan elapsed_time = stop_time - start_time;
            lblElapsedTime.Text = elapsed_time.TotalSeconds.ToString("0.00") + " seconds";
            this.Cursor = Cursors.Default;
        }

        // Perform an embossing transformation on the bitmap.
        public static Bitmap Emboss(Bitmap bm)
        {
            int[,] kernel =
            {
                {-1, 0, 0},
                {0, 0, 0},
                {0, 0, 1}
            };
            int weight = 1;
            return ApplyFilter(bm, kernel, weight);
        }

        // Apply a filter to the Bitmap.
        public static Bitmap ApplyFilter(Bitmap bm, int[,] kernel, int weight)
        {
            Bitmap new_bm = new Bitmap(bm.Width, bm.Height);
            int kernel_width = kernel.GetUpperBound(0) + 1;
            int kernel_height = kernel.GetUpperBound(1) + 1;
            int half_width = kernel_width / 2;
            int half_height = kernel_height / 2;

            for (int x = half_width; x <= bm.Width - 1 - half_width; x++)
            {
                for (int y = half_height; y <= bm.Height - 1 - half_height; y++)
                {
                    int r = 0, g = 0, b = 0;
                    for (int dx = 0; dx < kernel_width; dx++)
                    {
                        for (int dy = 0; dy < kernel_height; dy++)
                        {
                            Color clr = bm.GetPixel(
                                x + dx - 1,
                                y + dy - 1);
                            r += clr.R * kernel[dx, dy];
                            g += clr.G * kernel[dx, dy];
                            b += clr.B * kernel[dx, dy];
                        }
                    }

                    r = (int)(127 + r / weight);
                    g = (int)(127 + g / weight);
                    b = (int)(127 + b / weight);
                    if (r < 0) r = 0;
                    if (g < 0) g = 0;
                    if (b < 0) b = 0;
                    if (r > 255) r = 255;
                    if (g > 255) g = 255;
                    if (b > 255) b = 255;
                    new_bm.SetPixel(x, y, Color.FromArgb(r, g, b));
                }
            }
            return new_bm;
        }
    }
}
